"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
// available decoders
var codabar = require("./codabar");
var code128 = require("./code-128");
var code39 = require("./code-39");
var code93 = require("./code-93");
var code2of5 = require("./code2of5");
var ean13 = require("./ean-13");
var ean8 = require("./ean-8");
// utilities
var combineAllPossible_1 = require("./utilities/combineAllPossible");
var getImageDataFromSource_1 = require("./utilities/getImageDataFromSource");
var getLines_1 = require("./utilities/getLines");
var adaptiveThreshold_1 = require("./utilities/threshold/adaptiveThreshold");
var applySimpleThreshold_1 = require("./utilities/threshold/applySimpleThreshold");
var isTestEnv = process && process.env.NODE_ENV === 'test';
var BARCODE_DECODERS;
(function (BARCODE_DECODERS) {
    BARCODE_DECODERS["code-128"] = "code-128";
    BARCODE_DECODERS["code-2of5"] = "code-2of5";
    BARCODE_DECODERS["code-39"] = "code-39";
    BARCODE_DECODERS["code-93"] = "code-93";
    BARCODE_DECODERS["ean-13"] = "ean-13";
    BARCODE_DECODERS["ean-8"] = "ean-8";
    BARCODE_DECODERS["codabar"] = "codabar";
})(BARCODE_DECODERS = exports.BARCODE_DECODERS || (exports.BARCODE_DECODERS = {}));
function isImageLike(object) {
    return object.data && object.width && object.height;
}
function javascriptBarcodeReader(_a) {
    var image = _a.image, barcode = _a.barcode, barcodeType = _a.barcodeType, options = _a.options;
    return __awaiter(this, void 0, void 0, function () {
        var decoder, useSinglePass, imageData, _b, width, height, data, channels, finalResult, sPoints, slineStep, rowsToScan, i, sPoint, start, end, dataSlice, lines, result;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    switch (barcode) {
                        case BARCODE_DECODERS.codabar:
                            decoder = codabar.decoder;
                            break;
                        case BARCODE_DECODERS['code-128']:
                            decoder = code128.decoder;
                            break;
                        case BARCODE_DECODERS['code-39']:
                            decoder = code39.decoder;
                            break;
                        case BARCODE_DECODERS['code-93']:
                            decoder = code93.decoder;
                            break;
                        case BARCODE_DECODERS['code-2of5']:
                            decoder = code2of5.decoder;
                            break;
                        case BARCODE_DECODERS['ean-13']:
                            decoder = ean13.decoder;
                            break;
                        case BARCODE_DECODERS['ean-8']:
                            decoder = ean8.decoder;
                            break;
                        default:
                            throw new Error("Invalid barcode specified. Available decoders: " + BARCODE_DECODERS + ".");
                    }
                    useSinglePass = isTestEnv || (options && options.singlePass) || false;
                    if (!isImageLike(image)) return [3 /*break*/, 1];
                    _b = image;
                    return [3 /*break*/, 3];
                case 1: return [4 /*yield*/, getImageDataFromSource_1.getImageDataFromSource(image)];
                case 2:
                    _b = _c.sent();
                    _c.label = 3;
                case 3:
                    imageData = _b;
                    width = imageData.width;
                    height = imageData.height;
                    data = imageData.data;
                    channels = data.length / (width * height);
                    finalResult = '';
                    // apply adaptive threshold
                    if (options && options.useAdaptiveThreshold) {
                        data = adaptiveThreshold_1.applyAdaptiveThreshold(data, width, height);
                    }
                    sPoints = [5, 6, 4, 7, 3, 8, 2, 9, 1];
                    slineStep = Math.round(height / sPoints.length);
                    rowsToScan = Math.min(5, height);
                    for (i = 0; i < sPoints.length; i += 1) {
                        sPoint = sPoints[i];
                        start = channels * width * Math.floor(slineStep * sPoint);
                        end = start + rowsToScan * channels * width;
                        dataSlice = data.slice(start, end);
                        if (!options || !options.useAdaptiveThreshold) {
                            dataSlice = applySimpleThreshold_1.applySimpleThreshold(dataSlice, width, height);
                        }
                        lines = getLines_1.getLines(dataSlice, width, rowsToScan);
                        if (lines.length === 0) {
                            if (useSinglePass)
                                throw new Error('Failed to extract barcode!');
                            continue;
                        }
                        result = decoder(lines, barcodeType);
                        if (useSinglePass)
                            return [2 /*return*/, result];
                        if (!result)
                            continue;
                        if (!result.includes('?'))
                            return [2 /*return*/, result];
                        finalResult = combineAllPossible_1.combineAllPossible(finalResult, result);
                        if (!finalResult.includes('?'))
                            return [2 /*return*/, finalResult];
                        if (i === sPoints.length - 1)
                            return [2 /*return*/, finalResult];
                    }
                    throw new Error('Failed to extract barcode!');
            }
        });
    });
}
exports.javascriptBarcodeReader = javascriptBarcodeReader;
//# sourceMappingURL=index.js.map