"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function applyAdaptiveThreshold(data, width, height) {
    var integralImage = new Array(width * height).fill(0);
    var channels = data.length / (width * height);
    var t = 0.15; // threshold percentage
    var s = Math.floor(width / 4); // bracket size
    var s2 = Math.ceil(s / 2);
    for (var i = 0; i < width; i += 1) {
        var sum = 0;
        for (var j = 0; j < height; j += 1) {
            var pureIndex = j * width + i;
            var index = pureIndex * channels;
            // greyscale
            var v = (data[index] + data[index + 1] + data[index + 2]) / 3;
            data[index] = v;
            data[index + 1] = v;
            data[index + 2] = v;
            sum += v;
            if (i === 0) {
                integralImage[pureIndex] = sum;
            }
            else {
                integralImage[pureIndex] = integralImage[pureIndex - 1] + sum;
            }
        }
    }
    // skip edge rows
    for (var i = 0; i < width; i += 1) {
        for (var j = 0; j < height; j += 1) {
            var pureIndex = j * width + i;
            var index = pureIndex * channels;
            // no. of pixels per window
            var x1 = i - s2;
            var x2 = i + s2;
            var y1 = j - s2;
            var y2 = j + s2;
            if (x1 < 0)
                x1 = 0;
            if (x2 >= width)
                x2 = width - 1;
            if (y1 < 0)
                y1 = 0;
            if (y2 >= height)
                y2 = height - 1;
            var count = (x2 - x1) * (y2 - y1);
            var sum = integralImage[y2 * width + x2] -
                integralImage[y1 * width + x2] -
                integralImage[y2 * width + x1] +
                integralImage[y1 * width + x1];
            var v = 255;
            if (data[index] * count < sum * (1 - t)) {
                v = 0;
            }
            data[index] = v;
            data[index + 1] = v;
            data[index + 2] = v;
        }
    }
    return data;
}
exports.applyAdaptiveThreshold = applyAdaptiveThreshold;
//# sourceMappingURL=adaptiveThreshold.js.map